﻿using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using AutoMapper;
using InternshipSystem.Api.Queries;
using InternshipSystem.Core;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [Route("internshipRegistration")]
    public class InternshipRegistrationController : ControllerBase
    {
        private InternshipDbContext Context { get; }
        private IMapper Mapper { get; }

        public InternshipRegistrationController(InternshipDbContext context, IMapper mapper)
        {
            Context = context;
            Mapper = mapper;
        }

        /// <summary>
        /// Validate and add filled internship registration form 
        /// </summary>
        /// <param name="registrationQuery">Internship registration data</param>
        /// <response code="200">If registration form was successfully added</response>
        /// <response code="400">If the provided registration query was malformed</response>
        /// <response code="401">This action is only available for authorized student registered for current edition</response>
        [HttpPut]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        public async Task<ActionResult> SubmitRegistrationForm([FromBody] RegistrationFormQuery registrationQuery,
            CancellationToken cancellationToken)
        {
            var validator = new RegistrationFormQuery.Validator();
            var validationResult = await validator.ValidateAsync(registrationQuery, cancellationToken);

            if (!validationResult.IsValid)
            {
                return BadRequest(validationResult.ToString());
            }
            
            var internship = await GetCurrentStudentInternship();

            var internshipRegistration = internship.InternshipRegistration;

            if (registrationQuery.Company != null)
            {
                var company = registrationQuery.Company.Id.HasValue
                    ? await Context.Companies.SingleAsync(c => c.Id == registrationQuery.Company.Id,
                        cancellationToken: cancellationToken)
                    : Company.CreateCompany(registrationQuery.Company.Nip, registrationQuery.Company.Name);

                internshipRegistration.UpdateCompany(company);
            }

            var officeForm = registrationQuery.BranchOffice;
            if (officeForm != null)
            {
                BranchOffice branch;
                
                if (officeForm.Id.HasValue)
                { 
                    branch = await Context.Entry(internshipRegistration.Company)
                            .Collection(c => c.Branches)
                            .Query()
                            .SingleAsync(o => o.Id == officeForm.Id, cancellationToken: cancellationToken);
                }
                else
                {
                    branch = BranchOffice.CreateBranch(officeForm.Country, officeForm.City, officeForm.PostalCode,
                        officeForm.Street, officeForm.Building);
                    internshipRegistration.Company.AddBranchOffice(branch);
                }
                
                internshipRegistration.UpdateBranch(branch);
            }

            internshipRegistration.Start = registrationQuery.Start ?? internshipRegistration.Start;
            internshipRegistration.End = registrationQuery.End ?? internshipRegistration.End;

            var editionId = Guid.Parse(User.Claims.First(c => c.Type == "Edition").Value);
            
            var edition = await Context.Editions
                .FirstAsync(e => e.Id == editionId, cancellationToken);

            if (registrationQuery.Type.HasValue && edition.IsInternshipTypeAllowed(registrationQuery.Type.Value))
            {
                return BadRequest("Edycja nie posiada takiego rodzaju zatrudnienia w dostępnych zatrudnieniach");
            }
            
            internshipRegistration.Type = registrationQuery.Type ?? internshipRegistration.Type;
           
            await Context.SaveChangesAsync(cancellationToken);
            return Ok();
        }

        private async Task<Internship> GetCurrentStudentInternship()
        {
            // TODO: rewrite when authentication will be implemented
            var edition = await Context
                .Editions
                .FirstAsync();

            return await Context.Entry(edition)
                .Collection(e => e.Internships)
                .Query()
                .Include(i => i.InternshipRegistration)
                .FirstAsync();
        }
    }
}